; Copyright (c) 2005, Stubbe F. Hviid
;   All rights reserved
;+
; NAME:
;   p_resolve_database_file
;
; PURPOSE:
;   Resolve the location of a database file required by the osiris calibration pipeline.
;   The function expected the datafiles to be in the form: <camera>_<unit>_<fileid>_V<version>_<timestamp>.<fileext>
;                                                          Example:  OSINAC_FM_FLAT-FFPVIS-ORANGE_V2_16062005.img
;   Note that the <xxx> field cannot contain the character '_' since the underscore character is used as seperator character.
;
; CATEGORY:
;   calibration database management
;
; CALLING SEQUENCE:
;   FUNCTION p_resolve_database_file, subnode, camera, model, fileid, version, file_ext
;
;
; INPUTS:
;       subnode:        What subfolder pf the database tree is to be used
;       camera:         What camera (OSINAC or OSIWAC)
;       model:          What hardware model is to be used (FM, FS, QM ... )
;       fileid:         What file id is to be used
;
; OPTIONAL INPUTS:
;       version:        The file version to be used (If not specified the highest version will automatically be returned)
;       file_ext:       What file extension is to be used (defaults to first file matching .*)
;                       The file extension should NOT include the heading '.' character. example: '.img' should be specified as 'img'
;
; KEYWORD PARAMETERS:
;   None
;
; OUTPUTS:
;   Returns the path to fully qualified file on success, empty string on error
;
; OPTIONAL OUTPUTS:
;   None
;
; COMMON BLOCKS:
;   None
;
; SIDE EFFECTS:
;   None
;
; RESTRICTIONS:
;   None
;
; EXAMPLE:
;  path = p_resolve_database_file('flatfields', 'OSINAC', 'FM', 'FLAT-FFPVIS-ORANGE', '', 'img')
;
; MODIFICATION HISTORY:
;   Written by:  Stubbe F. Hviid  June 2005
;-

FUNCTION p_resolve_database_file, subnode, camera, model, file_id, version, file_ext

        ; handle default values
        if n_elements(subnode) eq 0 then subnode = ''
        if n_elements(camera) eq 0 then return, ''
        if n_elements(model) eq 0 then return, ''
        if n_elements(file_id) eq 0 then return, ''
        if n_elements(version) eq 0 then version = '*'
        if version eq '' then version = '*'
        if n_elements(file_ext) eq 0 then file_ext = '*'
        if file_ext eq '' then file_ext = '*'

        ; build searchmask
        mask = subnode + '/' + camera + '_' + model + '_' + file_id + '_V' + strtrim(version, 2) + '_*.' + file_ext

        ; Get the Osiris environment
        env = osiris_environment();

        ; search for files matching <mask>
        files = findfile(env.database_path + '/' + mask)

		; handle the simple return cases
		if n_elements(files) eq 0 then begin
			print, 'Parameter file not found: subnode=' + strtrim(subnode, 2) + ' camera=' + strtrim(camera, 2) + ' model=' + strtrim(model, 2) + ' version=' + strtrim(version, 2)
			return, ''		; no file found
		endif
		if n_elements(files) eq 1 then return, files[0] ; only one file found

		; multible files found, then find the file with the highest version number

		vers = intarr(n_elements(files))

		for i=0,n_elements(files)-1 do begin
			p = strpos(files[i], '_V', /REVERSE_SEARCH)
			if p gt 0 then begin
				vers[i] = fix(strmid(files[i], p+2))
			endif else begin
				vers[i] = 10000  ; just som every high number to indicate that no version field was found
				print, 'WARNING: No version field was found in file: ' + files[i]
			endelse
		endfor

		; now sort the file list by version number
		s = sort(vers)

		files = files[s]

		return, files[n_elements(files)-1]
END
